/*
 * @Description:
 * @Author: zhifeng.zhang
 * @Date: 2019-08-26 15:16:03
 * @LastEditors  : zhifeng.zhang
 * @LastEditTime : 2019-12-21 08:03:21
 */

function dateFormat(date, fmt) {
  const o = {
    'M+': date.getMonth() + 1, // 月份
    'd+': date.getDate(), // 日
    'h+': date.getHours(), // 小时
    'm+': date.getMinutes(), // 分
    's+': date.getSeconds(), // 秒
    'q+': Math.floor((date.getMonth() + 3) / 3), // 季度
    S: date.getMilliseconds() // 毫秒
  };

  if (/(y+)/.test(fmt)) {
    fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
  }

  for (const k in o) {
    if (new RegExp('(' + k + ')').test(fmt)) {
      if (k === 'S') {
        fmt = fmt.replace(RegExp.$1, (('000' + o[k]).substr(('' + o[k]).length)));
      } else {
        fmt = fmt.replace(
          RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)));
      }
    }
  }

  return fmt;
};

function stopServer(nodeChild, mongoChild) {
  if (nodeChild) {
    console.log('cleaning up node child');
    nodeChild.kill('SIGTERM');
    nodeChild = null;
  }

  if (mongoChild) {
    console.log('cleaning up mongo child');
    mongoChild.kill('SIGTERM');
    mongoChild = null;
  }

  const os = require('os');
  if (os.platform() === 'linux') {
    try {
      const spawn = require('child_process').spawn;
      spawn('killall', ['-15', 'mongod']);
    } catch (e) {
    }
  }
};

function deleteFolderRecursive(path) {
  const fs = require('fs');
  if (fs.existsSync(path)) {
    fs.readdirSync(path).forEach(function (file) {
      const curPath = path + '/' + file;
      if (fs.statSync(curPath).isDirectory()) { // recurse
        deleteFolderRecursive(curPath);
      } else { // delete file
        fs.unlinkSync(curPath);
      }
    });
    fs.rmdirSync(path);
  }
};

async function getWindowsDBDiskAvailable(dbConfigData, addLog) {
  if (!dbConfigData || !dbConfigData.db || dbConfigData.db.type === 'remote') {
    return -1;
  }
  const diskinfo = require('./programs/server/npm/node_modules/diskinfo');
  const currentDisk = dbConfigData.db.path.substr(0, 2).toLowerCase();
  addLog('Info', 'currentDisk: ' + currentDisk);
  let available = -1;
  return new Promise((resolve, reject) => {
    diskinfo.getDrives(function (err, aDrives) {
      if (err) {
        addLog('error', 'get disk error:' + err);
        resolve(available);
      }
      for (let i = 0; i < aDrives.length; i++) {
        addLog('Info', 'aDrives[i].mounted: ' + aDrives[i].mounted);
        addLog('Info', 'aDrives[i].available: ' + aDrives[i].available);
        if (aDrives[i].mounted.toLowerCase() === currentDisk) {
          available = (aDrives[i].available / 1024 / 1024 / 1024).toFixed(0);
        }
      }
      resolve(available);
    });
  });
};

async function getlinuxDBDiskAvailable(dbConfigData, addLog) {
  if (!dbConfigData || !dbConfigData.db || dbConfigData.db.type === 'remote') {
    return -1;
  }
  return new Promise((resolve, reject) => {
    const spawn = require('child_process').spawn;
    const dfChild = spawn('df', ['-l', dbConfigData.db.path]);

    dfChild.stderr.setEncoding('utf8');
    dfChild.stderr.on('data', function (nodeData) {
      console.log('stderr: ', nodeData);
      addLog('Info', 'dfChild: ' + nodeData);
      resolve(-1);
    });

    dfChild.stdout.setEncoding('utf8');
    addLog('Info', 'dfChild.stdout.on');
    dfChild.stdout.on('data', function (nodeData) {
      addLog('Info', 'dfChild: ' + nodeData);
      const diskList = nodeData.split('\n');
      if (diskList.length < 2) {
        console.log('Info', 'read disk info error.');
        resolve(-1);
      }
      const tmpList = diskList[1].split(' ');
      const dfList = [];
      for (let i = 0; i < tmpList.length; i++) {
        if (tmpList[i] !== '') {
          dfList.push(tmpList[i]);
        }
      }
      if (dfList.length < 4) {
        console.log('Info', 'read disk info error.');
        resolve(-1);
      }
      let available = dfList[3];
      available = (available / 1024 / 1024 / 2).toFixed(0);
      resolve(available);
    });
  });
};

async function startMeteor(infoLabel, deteilLabel) {
  deteilLabel.innerHTML = 'Starting meteor server...';
  const childProcess = require('child_process');
  const os = require('os');
  const fs = require('fs');
  const path = require('path');
  // var nw = require('nw.gui')
  let appName = 'Archon';
  const dirname = process.cwd();
  const loadWin = nw.Window.get();
  // if (loadWin) {
  //   loadWin.maximize()
  // }

  const packageJson = require(path.resolve(dirname, './package.json'));
  if (Object.hasOwnProperty.call(packageJson, 'name')) {
    appName = packageJson.name;
  }

  let appPath = '';
  let dataPath = '';
  let bundlePath = '';
  if (os.platform() === 'darwin') {
    appPath = path.join(process.env.HOME, 'Library/Application Support/', appName, '/');
  } else if (os.platform() === 'win32') {
    appPath = path.join(process.env.HOMEDRIVE, process.env.HOMEPATH, 'AppData/Local/', appName, '/');
  } else if (os.platform() === 'linux') {
    appPath = path.join(process.env.HOME, '/.config/', appName, '/');
  }
  dataPath = path.join(appPath, 'data');
  if (Object.hasOwnProperty.call(packageJson, 'dataVersion') && packageJson.dataVersion !== '') {
    dataPath = path.join(appPath, 'data_' + packageJson.dataVersion);
  }
  bundlePath = path.join(appPath, 'bundle');
  let logPath = path.resolve(dirname, 'log');
  if (os.platform() === 'darwin') {
    logPath = path.join(appPath, 'log');
  }

  if (os.platform() === 'linux') {
    try {
      console.log('before kill mongod: ');
      const spawn = require('child_process').spawn;
      spawn('killall', ['-15', 'mongod']);
      console.log('kill mongod success: ');
    } catch (e) {
      console.log('kill mongod fail: ');
    }
    const dataLockPath = path.join(dataPath, 'journal');
    if (fs.existsSync(dataLockPath)) {
      deleteFolderRecursive(dataLockPath);
    }
  }

  // If file paths don't yet exist, create them
  if (!fs.existsSync(appPath)) {
    fs.mkdirSync(appPath);
  }

  if (!fs.existsSync(dataPath)) {
    fs.mkdirSync(dataPath);
  }

  if (!fs.existsSync(bundlePath)) {
    fs.mkdirSync(bundlePath);
  }

  if (!fs.existsSync(logPath)) {
    fs.mkdirSync(logPath);
  }

  logPath = path.join(logPath, dateFormat(new Date(), 'yyyyMMdd') + '.server.log');
  if (!fs.existsSync(logPath)) {
    fs.writeFileSync(logPath, '');
  }

  function addLog(type, data) {
    fs.appendFileSync(logPath, `[${dateFormat(new Date(), 'hh:mm:ss.S')}] [${type}] - ${data}\r\n`);
  }

  async function start(callback) {
    if (process.env.NODE_ENV === 'development') {
      console.log('Running in Dev mode.');
      const debugURL = 'http://127.0.0.1:3000';
      callback(debugURL);
    } else {
      addLog('Info', '***************************Start*****************');
      addLog('Info', 'Starting production server');
      addLog('Info', JSON.stringify(packageJson, '', 4));
      deteilLabel.innerHTML = 'Starting mongo server...';

      // Path to mongod command bundled with app.
      let mongodPath = path.join(dirname, 'resources', 'mongod');
      let nodePath = path.join(dirname, 'resources', 'node');
      if (os.platform() === 'win32') {
        mongodPath = path.join(dirname, 'resources', 'mongod.exe');
        nodePath = path.join(dirname, 'resources', 'node.exe');
      }

      try {
        if (!fs.existsSync(mongodPath) || !fs.existsSync(nodePath)) {
          addLog('Error', 'mongod or node not exists');
          infoLabel.innerHTML = 'Initialization failed';
          deteilLabel.innerHTML = 'mongod or node not exists';
          deteilLabel.style.color = 'red';
          return;
        }
        try {
          fs.accessSync(mongodPath, fs.constants.X_OK);
        } catch (error) {
          addLog('Info', 'chmod 755 mongod');
          fs.chmodSync(mongodPath, '755');
        }

        try {
          fs.accessSync(nodePath, fs.constants.X_OK);
        } catch (error) {
          addLog('Info', 'chmod 755 mongod');
          fs.chmodSync(nodePath, '755');
        }
      } catch (error) {
        addLog('Error', 'chmod mongod or node failed!');
        infoLabel.innerHTML = 'Initialization failed';
        deteilLabel.innerHTML = error;
        deteilLabel.style.color = 'red';
        return;
      }

      const configPath = path.join(appPath, 'config.json');
      let configData = null;
      const mongoPort = 37432;
      addLog('Info', 'config path: ' + configPath);
      if (!fs.existsSync(configPath)) {
        configData = {
          db: {
            type: 'local',
            host: '127.0.0.1',
            port: mongoPort,
            path: dataPath,
            freedisk: 5,
            storagedays: 1000
          }
        };
        fs.writeFileSync(configPath, JSON.stringify(configData, null, 2));
      } else {
        const res = fs.readFileSync(configPath, { encoding: 'utf8' });
        configData = JSON.parse(res.toString());
        if (!configData.db) {
          configData.db = {
            type: 'local',
            host: '127.0.0.1',
            port: mongoPort,
            path: dataPath,
            freedisk: 5,
            storagedays: 1000
          };
        }
        // configData.db.path = dataPath;
        fs.writeFileSync(configPath, JSON.stringify(configData, null, 2));
        // addLog('Info', 'config path: ' + JSON.stringify(configData, null, 2));
        dataPath = configData.db.path;
      }
      let diskAvailable = -1;
      if (os.platform() === 'win32') {
        diskAvailable = await getWindowsDBDiskAvailable(configData, addLog);
      } else {
        diskAvailable = await getlinuxDBDiskAvailable(configData, addLog);
      }
      addLog('Info', 'getWindowsDBDiskAvailable: ' + diskAvailable);
      if (diskAvailable !== -1 && diskAvailable < configData.db.freedisk) {
        infoLabel.innerHTML = 'Check free disk';
        deteilLabel.innerHTML = 'The disk free space of database must be greater than ' + configData.db.freedisk + 'G';
        deteilLabel.style.color = 'red';
        return;
      }

      function startNode(configData, mongoChild, nodePath, callback) {
        const webPort = 37000;
        deteilLabel.innerHTML = 'Starting node server...';
        addLog('Info', 'Starting node child...');
        addLog('Info', 'WebPort-script: ' + webPort);
        console.log('Starting node child...');
        const rootURL = 'http://127.0.0.1';
        const userEnv = process.env;
        userEnv.ROOT_URL = rootURL;
        userEnv.PORT = webPort;
        userEnv.BIND_IP = '127.0.0.1';
        userEnv.DB_PATH = configData.db.path;
        userEnv.MONGO_URL = 'mongodb://' + configData.db.host + ':' + configData.db.port + '/meteor';
        // user_env.METEOR_SETTINGS = fs.readFileSync(path.join(dirname, 'resources', 'settings.json'), 'utf8');
        userEnv.DIR = dirname;
        userEnv.NODE_ENV = 'production';
        userEnv.NODE_PATH = path.join(dirname, 'programs', 'server', 'npm', 'node_modules');
        addLog('Info', 'Starting node mongo ' + userEnv.MONGO_URL);
        const nodeArgs = path.join(dirname, 'main.js');
        let memory = 2048;
        if (packageJson.window.memory) {
          memory = packageJson.window.memory;
        }
        memory = '--max-old-space-size=' + memory;
        const nodeChild = childProcess.spawn(nodePath, [memory, nodeArgs], {
          env: userEnv
        });

        let opened = false;
        // listen for errors
        nodeChild.stderr.setEncoding('utf8');
        nodeChild.stderr.on('data', function (nodeData) {
          console.log('stderr: ', nodeData);
          addLog('Info', 'nodeChild: ' + nodeData);
        });

        nodeChild.stdout.setEncoding('utf8');
        addLog('Info', 'nodeChild.stdout.on');
        nodeChild.stdout.on('data', function (nodeData) {
          // console.log(nodeData);
          addLog('Info', 'nodeChild: ' + nodeData);
          // addLog('Info', 'check start nodeChild:' + nodeData + '\r\n')
          if (nodeData.indexOf('Start meteor Server') !== -1) {
            if (!opened) {
              opened = true;
            } else {
              return;
            }
            setTimeout(function () {
              const fullURL = rootURL + ':' + webPort;
              callback(fullURL, nodeChild, mongoChild);
            }, 100);
          }
        });
      }
      addLog('Info', 'configData: ' + JSON.stringify(configData, null, 2));
      if (configData.db.type !== 'remote') {
        const mongoLock = path.join(dataPath, 'mongod.lock');
        if (fs.existsSync(mongoLock)) {
          fs.unlinkSync(mongoLock);
        }
        // start mongo
        // Arguments passed to mongod command.
        addLog('Info', 'MongoPort-script: ' + configData.db.port);
        let mongodArgs = ['--bind_ip', '0.0.0.0', '--dbpath', dataPath, '--port', configData.db.port, '--unixSocketPrefix', configData.db.path, '--journal'];

        if (os.platform() === 'win32') {
          mongodArgs = ['--bind_ip', '0.0.0.0', '--dbpath', dataPath, '--port', configData.db.port, '--journal'];
        } else if (os.platform() === 'darwin') {
          mongodArgs = ['--bind_ip', '0.0.0.0', '--dbpath', dataPath, '--port', configData.db.port, '--journal'];
        }

        addLog('Info', 'Start run mongo process...');
        // Start the Mongo process.
        // Be sure to change the PURPOSE to be the name of your app
        const mongoChild = childProcess.spawn(mongodPath, mongodArgs, {
          env: {
            PURPOSE: `MY_${appName}_APP`
          }
        });

        mongoChild.stderr.setEncoding('utf8');
        mongoChild.stderr.on('data', function (data) {
          addLog('Error', 'mongoChild: ' + data);
        });

        let started = false;

        mongoChild.stdout.setEncoding('utf8');
        mongoChild.stdout.on('data', function (data) {
          addLog('Info', 'mongoChild: ' + data);
          let compareStr = 'waiting for connections on port ' + configData.db.port;
          if (os.platform() === 'darwin') {
            compareStr = 'Waiting for connections';
          }
          if (data.indexOf(compareStr) !== -1) {
            if (!started) {
              started = true;
            } else {
              return;
            }
            startNode(configData, mongoChild, nodePath, callback);
          }
        });
      } else {
        addLog('Info', 'start node 2');
        startNode(configData, null, nodePath, callback);
      }
    }
  }
  try {
    await start(function (url, nodeChild, mongoChild) {
      try {
        console.log('App occupying ', url);
        addLog('Info', 'start ' + url);
        // const manifest = nw.App.manifest.window;
        nw.Window.open(url, {}, function (curWin) {
          let checkWin = setInterval(() => {
            addLog('Info', 'open ' + url);
            if (curWin) {
              addLog('Info', 'curwin:' + JSON.stringify(curWin));
            } else {
              addLog('Warn', 'curwin: is null');
            }
            if (curWin) {
              try {
                curWin.window.NW_Open_Url = url;
                curWin.window.NW_closeHandle = [() => {
                  addLog('Info', 'win close');
                  stopServer(nodeChild, mongoChild);
                }];
                curWin.on('closed', function () {
                  curWin = null;
                  addLog('Info', 'closed');
                  // stopServer(nodeChild, mongoChild);
                });
                curWin.maximize();

                loadWin.close(true);
                addLog('Info', 'open finished');
              } catch (error) {
                addLog('Error', error.toString());
                stopServer(nodeChild, mongoChild);
                infoLabel.innerHTML = 'Open window failed';
                deteilLabel.innerHTML = error;
                deteilLabel.style.color = 'red';
              }
              clearInterval(checkWin);
              checkWin = null;
            }
          }, 1000);
        });
      } catch (error) {
        stopServer(nodeChild, mongoChild);
        infoLabel.innerHTML = 'Open window failed';
        infoLabel.style.color = 'red';
        deteilLabel.innerHTML = error;
        deteilLabel.style.color = 'red';
      }
    });
  } catch (error) {
    infoLabel.innerHTML = 'Initialization failed';
    deteilLabel.innerHTML = error;
    deteilLabel.style.color = 'red';
  }
};

// try {
//   startMeteor()
// } catch (error) {
//   alert(error)
// }
