# Archon User Guide

---
<!-- TOC -->
- [Archon User Guide](#archon-user-guide)
  - [1 Introduction](#1-introduction)
  - [2 Install](#2-install)
  - [3 Quick Start](#3-quick-start)
    - [3.1 Create Project](#31-create-project)
    - [3.2 Configure Instrument](#32-configure-instrument)
    - [3.3 Write Test Case](#33-write-test-case)
    - [3.4 Write Test Plan](#34-write-test-plan)
    - [3.5 Debugging](#35-debugging)
    - [3.6 Publish Project](#36-publish-project)
  - [4 Pin Point Configuration](#4-pin-point-configuration)
    - [4.1 View](#41-view)
    - [4.2 Pinpoint Configuration File Path](#42-pinpoint-configuration-file-path)
    - [4.3 Writing  Pinpoint Configuration File](#43-writing-pinpoint-configuration-file)
      - [4.3.1 Background Description](#431-background-description)
        - [4.3.1.1 Make Pinpoint Background Image](#4311-make-pinpoint-background-image)
        - [4.3.1.2 Export Background Image From Schematic](#4312-export-background-image-from-schematic)
      - [4.3.2 DefaultConfig Description](#432-defaultConfig-description)
      - [4.3.3 Bit_Symname_Map Description](#433-bit-symname-map-description)
      - [4.3.4 Points](#434-points)
        - [4.3.2.1 When The Needle Point Has Only A BIT Bit](#4321-when-the-needle-point-has-only-a-BIT-bit)
        - [4.3.2.2 The Case Where The Pin Point Contains The Function Point And The BIT Bit](#4322-the-case-where-the-pin-point-contains-the-function-point-and-the-BIT-bit)
        - [4.3.2.3 Needle Point Contains Function Point Nesting](#4323-needle-point-contains-function-point-nesting)
    - [4.4 Write Pinpoint Table File](#44-write-pinpoint-table-file)
    - [4.5 Import Needle Point Information](#45-import-needle-point-information)
    - [4.6 Configure Pinpoint Information](#46-configure-pinpoint-information)
      - [4.6.1 Select The Pin Point That Needs To Be Configured](#461-select-the-pin-point-that-needs-to-be-configured)
      - [4.6.2 Configured Pin Point](#462-configured-pin-point)
        - [4.6.2.1 Configure The Bits Of The BIT](#4621-configure-the-bits-of-the-BIT)
        - [4.6.2.2 Configured Function Points](#4622-configured-function-points)
    - [4.7 Export Pin Point Information](#47-export-pin-point-information)
    - [4.7 Reset To Default](#47-reset-to-default)
    <!-- /TOC -->

---

## 1 Introduction

Archon is a hardware-oriented test management tool for full scenario, which is suitable for debugging and verification in hardware development stage, debugging in NPI stage, automatic testing in production stage and other scenarios, aiming to reduce the difficulty for test engineers to develop, manage and run test cases.

Archon develops test case and test plan in a low-code way, supports more complex test cases through Python, and extends system function by developing plug-ins. Archon allows you to view test results and data in real time and export data using report templates. You can customize rport templates to meet specific data export requirements.  

![archon_platform](./resource/en/v2.3/Archon_Platform.png)

As shown in the figure above, Archon mainly consists of UI and Core, which are extended by plug-ins. UI provides GUI such as test case and test plan writing, test result display, user authority management and data report export. The main functions of the Core include the scheduling and operation of test cases, the collection of test result data, plug-in management and communication link maintenance.

## 2 Install

The Archon installation file is a zip compressed package, such as Archon-windows-V2.1.zip. Unzip Archon-windows-V2.1.zip, enter the **Archon-windows-V2.1/ui** directory, and double-click **Archon.exe** to run. Archon can only be used after registration. If Archon is not registered after startup, the following information will appear. Please provide the machine code shown in the red box in the figure to our staff to obtain the registration cdoe. Get the registration code, fill in the input box and click the **OK** button.

![3_license](./resource/en/v2.3/archon_no_license.png)

## 3 Quick Start

This document takes the control instrument to measure a volatage as an example to describe how to connect the instrument, how to write and debug test case and test plan.

### 3.1 Create Project

3.3.1 Click the **Project** button in the left navigation bar, and then click the **New project** button to create a new project.

![new_project_1](./resource/en/v2.3/project_guide.png)

3.1.2 Fill in the project information, and click **OK**.

![new_project_2](./resource/en/v2.3/new_project_config.png)

- **Project name**: Project name
- **Version(Option)**: Project version
- **Description(Option)**: Project description information
- **Report type(Option)**: Type of test report automatically generated, The *Archon Single Report* means that one report is produced for each test, and the *Archon Total Report* means that multiple tests are combined into one report.
- **Test strategy**: running mode, *Serial* is sequential execution, *Paraller* is parallel execution.

### 3.2 Configure Instrument

3.2.1 Before configuration, connect the instrument to the PC, and then click the *Instruments* button in the navigation bar on the left side of project management (as shown by arrow 1 in the figure below) or the instrument status button (as shown by arrow 2 in the figure below) to enter the instrument management page.

![add_device_1](./resource/en/v2.3/project_instruments.png)

3.2.2 Click *Add* button(as show by arrow 3 in the figure above) to add new instrument.

![add_device_2](./resource/en/v2.3/Add_instrument_Blank.png)

- **Name**: Instrument name
- **Vendor**: Instrument manufacturer, it supports drop-down list selection and manual input.
- **Partno**: Instrument model, it supports drop-down list selection and manual input.
- **SN**: Instrument serial number
- **Channel**: Test channel，a test channel can be understood simple as a test thread. Archon can support multiple channels to run at the same time.
- **Connect type**: The following connection modes are supportd:
  - VISA: Virtual Instrument Software Architecture
  - SERIAL
  - TcpClient
  - TcpServer
  - SSH
  - Arbiter: SmartGiant controller interface
  - Plugin: Python based plug-in
- **Command Terminator**: Command terminator，A command string sent to the instrument must terminate with the character.
- **Identify Command**: Identify command，default is *IDN?
- **Identify String**: Identify string，this configration, together with the identify command, is used for instrument lookup adaptation. Identify default is "\<Manufacturer>,\<model number>". For example "Agilent Technologies,33519B".
Automatic instrument adaptation means that after Archon opens the project, it automaitcally scans all instruments connected with Archon and sends identify command to the instruments. If the information returned by the identify command is the same as one of the identify string list, the instrument is considered to be correctly matched and automatically connect.

3.2.3 In the new instrument page, fill in the instrument name, select the manufacturer and model, select VISA for *Connect type*(as show by arrow 1 in the figure below), select the instrument for *VISA ID*(as show by arrow 2 in the figure below), and use the default configuration for other.

![add_device_3](./resource/en/v2.3/project_add_device_visa.png)

3.2.4 In the new instrument interface, fill in the instrument name, select the manufacturer: SmartGiant and the model: Arbiter, and fill in the IP address.

![add_device_4](./resource/en/v2.3/project_add_arbiter.png)

3.2.5 Add a VISA instrument named Power according to the method in 3.2.3.

3.2.6 After adding the instrument, click the **OK** button to save the configuration. The connection status of the instrument can be seen in the instrument list.
![add_device_5](./resource/en/v2.3/project_instruments_ok.png)

3.2.7 After the saving is complete, click the **Pin map** button in the instrument list (as shown by arrow 1 in the figure below) to configure the pinpoint information of the Arbiter controller.
![add_device_6](./resource/en/v2.3/project_set_arbiter_bit.png)
![add_device_6](./resource/en/v2.3/project_set_arbiter_bit2.png)

### 3.3 Write Test Case

3.3.1 Click the **Test case** button in the navigation bar on the left of the project management page to enter the **Test case** page, and click the add button(as show by arrow 1 in the figure below) to create a test case named **Testcase1**.

![add_testcase_1](./resource/en/v2.3/project_testcase_blank.png)

3.3.2 Select the test case, right-click and select **Rename** to rename the test case name as set_input_voltage.

![add_testcase_2](./resource/en/v2.3/project_testcase_menu.png)

3.3.3 Use VSCode to create a Python file, and write the following code, save as **utility.py**.

```py
def strtofloat(str):
    return float(str)
```

3.3.4 Click **Import** button(as show by arrow 1 in the figure below), select **utility.py** and click **Open**.

![tc_rename](./resource/en/v2.3/project_testcase_import_tpy.png)

![tc_rename](./resource/en/v2.3/project_testcase_tpy.png)

3.3.5 Select **set_input_voltage** and writ code in the right table.

![tc_rename](./resource/en/v2.3/project_testcase_function_list.png)
![tc_rename](./resource/en/v2.3/project_testcase_addon_function_list.png)
![tc_rename](./resource/en/v2.3/project_testcase_addon_params_list.png)

![tc_rename](./resource/en/v2.3/project_testcase_info.png)

Table header description:

- Device: Device name to be used
- Function: Function name of device support
- Params: The function parameter
- Value Name: The alias of the test case result, which is convenient for later use.
- Value Unit: The result unit.
- Low Limit: Low limmit value.
- High Limit: High limmit value.
- Timeout: Unit is millisecond, default is infinite.

Code description:

- Line 1: Set voltage to 22V. The parameter is SCPI command, please refer to the user manual fo the instrument.
- Line 2: Delay 100 millisecond.
- Line 3: Read voltage and save it to input_vol.
- Line 4：Convert result to numbers.

3.3.6 Create a new test case **get_eload_current**.

![tc_rename](./resource/en/v2.3/project_testcase_info_1.png)

3.3.7 Create a new test case **get_eload_voltage**.

![tc_rename](./resource/en/v2.3/project_testcase_info_2.png)

3.3.8 Click **Save** button to save test cases.

### 3.4 Write Test Plan

3.4.1 Click the **Test plan** button in the navigation bar on the left of the project management page to enter the test plan page, and click the **Add** button(as show by arrow 1 in the figure below) to create a new test plan.

![add_testplan_1](./resource/en/v2.3/project_testplan.png)

3.4.2 In the add test plan page, input the name and bind the channel, click the **OK** button to create a new test plan.

![add_testplan_2](./resource/en/v2.3/project_testplan_new.png)

3.4.3 Select **Test** file, right-click in the right table, select **Insert 1 rows** to add a new row.

![add_testplan_2](./resource/en/v2.3/project_testplan_edit.png)

3.4.4 Double click the blank row in the **Case Name** column and select the set_inpurt_voltage from the drop-down list, and click **Save** button to save.

![add_testplan_3](./resource/en/v2.3/project_testplan_edit_2.png)

### 3.5 Debugging

The **Test plan** page consists of five parts:

- Run and debug button, as show in the red box 1 below.
- Data view, as show in the red box 2 below.
- Configure, as show in the red box 3 below.
- Data statistics, as show in the red box 4 below.
- Log view, as show in the red box 5 below.

![debug](./resource/en/v2.3/project_testplan_viewer.png.png)

3.5.1 The debugging button is shown in the figure below.

![debug_1](./resource/en/v2.3/project_testplan_debug_button.png)

- Run/Pause
- Stop: Stop run
- Step: Run step by step

3.5.2 Set breakpoint, click on the far left(as show in the red box below) of the test plan table to set a breakpoin.

![debug_2](./resource/en/v2.3/project_testplan_breakpoint.png)

3.5.3 Data view: As shown in the figure below, click the arrow 1 button to display the detail of the test case in this line, arrow 2 shows the status of the test case, arrow 3 shows the result and status of each line of the test case.

![debug_3](./resource/en/v2.3/project_testplan_table_info.png)

3.5.4 Configure：

![debug_4](./resource/en/v2.3/project_testplan_run_settings.png.png)

- Stop test when running error: If the test result if Fail, system will stop running.
- Loop test: Loop test, configurable loop times.

3.5.5 Data Statistics：

![debug_5](./resource/en/v2.3/project_testplan_run_totol.png)

- Statistics: Count the number of successes and failures of the current project.
- Channel information: Count the number of successes and failures of the current channel.

3.5.6 Data View:

![debug_6](./resource/en/v2.3/project_testplan_log.png)

- Time: Logging time
- Type: Logging level
- Channel: Channel name
- TestCase: For example, **2-1** represents the log for the first line of the second test case in the test plan.
- Result: Test result
- Detail: Detail log, click will open detail log page.

![debug_7](./resource/en/v2.3/project_testplan_log_detail.png)

3.5.7 If you find bugs in Archon, please describe them in detail and send the logs of the following paths back to us as attachments, we will solve them as soon as possible.

- Core log path: $Archon_install_path/core/log/
- UI log path: $Archon_install_path/ui/log/

### 3.6 Publish Project

3.6.1 After debugging, you need to publish the project before it can be used in the production environment. Click **Publish** on the project management page to enter the project publish page, fill in the version, select the storage path, fill in the name, and click **Publish** button to publis the project.

![publish_1](./resource/en/v2.3/project_publish.png)

3.6.2 After the project is released, the products need to be bound before they can be used in production. Click **Products** button on the main page enter the product manage page, and click **Add** button.

![publish_2](./resource/en/v2.3/products_add.png)

3.6.3 Fill in product model, and enter import page.

![publish_3](./resource/en/v2.3/products_add_import.png)

3.6.4 Click **Import** button, select the published project package, and click **OK** button.

![publish_4](./resource/en/v2.3/products_add_import_packages.png)

3.6.5 Click **ATS** on the main page enter production page, select the product and project package, and click **Start Test** to run the project.

![publish_5](./resource/en/v2.3/ATS_Run.png)

## 4 Pin Point Configuration
Pin point configuration is a unique configuration item of our instruments: Arbiter and MadagascarA. Pin point configuration refers to setting a series of BIT bits that need to be switched for the test function to be set into a set, and calling the switching instruction to fill in the set when writing a test case. The name is enough, which is very convenient for users to realize BIT switching.

### 4.1 View
There are two pinpoint views for pinpoint configuration: graphical view and table view
- Graphical View:
  ![point_1](./resource/en/v2.3/point_main_canvas.png)
  
- Table view:
![point_2](./resource/en/v2.3/point_main_table.png)


1. Archon defaults to the graphic view and the pinpoint configuration matches its background graphics. When the background image used in your project is different, you need to manually edit the graphics file and write the pinpoint configuration file to describe the pinpoint information.
2. If the table view is used, write a pin point configuration file to describe the pin point information and the corresponding table csv file.

### 4.2 Pinpoint Configuration File Path
The display mode of the pinpoint view is determined by the pinpoint configuration file of the project. The pinpoint configuration file path is: \$project path/controlbit/[\$first channel name]\$instrument name

- The project path refers to the user-created project or the path to open an existing project.
- The first channel name refers to the name of the first channel in the channel configuration, and the default is channel 1.
- Instrument name refers to the user-configured instrument name.

### 4.3 Writing  Pinpoint Configuration File

The pinpoint configuration file structure for the project is as follows:
```js
{
  "configs": {
    "background": {},
    "defaultConfig":{},
    "bit_symname_map":{},
    "points": [],
    "zoom":1
  }
}
```
#### 4.3.1 Background Description
The background structure is as follows:
```js
{
  "x": 0,
  "y": 0,
  "type": "image",
  "backgroundImg": "TP_MAP.png",
  "tableFile": ""
}
```
- type: The type of pin point view, image represents the graphic view, table represents the table view, if not filled in, the default is image.
- x,y: The image coordinates of the graphics view, the default is 0.
- backgroundImg: The file name of the graphic view image or the image base64 format data, the image conversion base64 tool address:https://tool.chinaz.com/tools/imgtobase，As shown below:
![point_3](./resource/en/v2.3/point_image_to_base64.png)  
- tableFile: The name of the configuration file for the table.

##### 4.3.1.1 Make Pinpoint Background Image
The key to making it yourself is the production of the background image. When making the background image, you need to consider the size and layout of the needle points.
The following example:
![point_4](./resource/en/v2.3/point_edit_image.png)

- The size of the planning square is 250px, 8 needle points are placed in the square, the radius of each needle point is 8px (the actual size is 16\*16), and the interval between each needle point is 14px, which means that each needle point is actually The occupied space is 16+14=30px. The actual space occupied by 8 needle points is 8\*30=240px, then the inner margin of the square is (250-240)/2 = 5px.
- The position of the first square is x=80px, y=80px, and the position of the first needle point in the first square is: </br>
   80(square X) + 5(padding) + 7(spacing 14px/2) + 8(center coordinate) = 100px </br>
   80(square Y) + 5(padding) + 7(spacing 14px/2) + 8(center coordinate) = 100px </br>
- Based on this, the needle point positions of the first row and second column of the first square are:</br>
   X: 100 (first column position) + 8 (first column radius) + 14 (spacer) + 8 (second column radius) = 130px </br>
   Y: 100px

##### 4.3.1.2 Export Background Image From Schematic
In the schematic diagram of the actual project, the size of the base map and the name, size and position information of each pin point should be included. The base map can be exported by selecting or masking layers.
- Export the basemap and adjust the basemap to a suitable size, as shown in the following figure:
  ![point_4](./resource/en/v2.3/point_import_image.png)

- The user needs to confirm the size and position of the needle point. Due to the difference between the unit in the schematic diagram and the pixel unit of the picture, after exporting the base map, it is necessary to compare the actual size of the picture pixel and the schematic diagram to obtain the conversion ratio.
  ![point_4](./resource/en/v2.3/point_image_size_change.png)

  As shown in the figure above, the base image in the schematic is 3CM wide, and the adjusted background image is 600px.
  Here, we roughly get the conversion formula: 600(px) = 300(mm) \* 2
  Through this formula, the size and position of the background image corresponding to each pin point can be obtained. As shown in the schematic diagram, the pin size is 8mm, the coordinates are (10,12)mm, the converted size is 8\*2=16px, and the coordinates are (20px, 24px)



#### 4.3.2 DefaultConfig Description
defaultConfig pin point graphics configuration (size and background color), optional, invalid for table view, the structure is as follows:
```js
{
  "defaultConfig" : {
    "height": 8,
    "width": 8,
    "state": [
      {
        "ID": "normal",
        "background": "#909399",
        "opacity": 1
      },
      {
        "ID": "config",
        "background": "#409EFF",
        "opacity": 1
      },
      {
        "ID": "selected",
        "background": "#0F0",
        "opacity": 1
      }
    ]
  },
}
```
- height: needle point length.
- width: stitch width.
- state: Pin state color configuration.
- ID indicates the status and cannot be modified, normal indicates that the pin point has no configuration (no function points or BIT bits), config has configuration (configured with function points or BIT bits), and selected indicates the selected state.
- background: pinpoint background color.
- opacity: reserved configuration.

#### 4.3.3 Bit_Symname_Map Description
bit_symname_map is optional. It is only valid when background.type is table. It is used to draw the sequence diagram of switch switching. The structure is as follows:
- key: name
- symname: switch label
- dir: switch direction

```js
{
  "bit_symname_map":{
    "K59,K85(MEAS)": [
        {
            "symname": "K59",
            "dir": "31"
        },
        {
            "symname": "K85",
            "dir": "31"
        }
    ],
    "K59,K85(RST)": [
        {
            "symname": "K59",
            "dir": "31"
        },
        {
            "symname": "K85",
            "dir": "31"
        }
    ],
    ......
  },
  "points":[
    {
      "name": "BT_I2S_DI_TO_SCOPE",
      "description": "BT_I2S_DI_TO_SCOPE MEASUREMEANT",
      "list": [
        {
          "name": "BT_I2S_DI_TO_SCOPE_MEAS",
          "description": "BT_I2S_DI_TO_SCOPE MEASUREMEANT",
          "list": [
            "K59,K85(MEAS)"
          ]
        },
        {
          "name": "BT_I2S_DI_TO_SCOPE_RST",
          "description": "BT_I2S_DI_TO_SCOPERESET",
          "list": [
            "K59,K85(RST)"
          ]
        }
      ]
    },
    {
       "name": "BT_I2S_DO_TO_SCOPE",
       "description": "BT_I2S_DO_TO_SCOPE MEASUREMEANT",
       "list": [
         {
           "name": "K59,K85(MEAS)",
           "description": "K59,K85 MEASUREMEANT",
           "list": [
              "K59,K85(MEAS)"
           ]
         },
         {
           "name": "K59,K85(RST)",
           "description": "K59,K85 RESET",
           "list": [
              "K59,K85(RST)"
           ]
         }
       ]
    },
    {
        "name": "BT_REG_ON_TO_DM01",
        "description": "BT_REG_ON__CONN_TO_DM01 MEASUREMEANT",
        "list": [
          {

            "name": "BT_REG",
            "description": "BT_REG_ON__CONN_TO_DM01 MEASUREMEANT",
            "list": [
              "K59,K85(MEAS)"
            ]
          },
          {
            "name": "BT_RST",
            "description": "BT_REG_ON__CONN_TO_DM01RESET",
            "list": [
              "K59,K85(RST)"
            ]
          }
        ]
    }
  ],
}
```
![point_3](./resource/en/v2.3/point_table_bitmap.png)
![point_4](./resource/en/v2.3/point_table_bitmap_info.png)
As shown in the figure above, the description of 1 is determined according to the name of the needle point. The naming rule is aa_TO_bb, and the end of the parsing is aa connect to bb.
As shown in the figure above, K59 and K85 of 2 are K59 that read bit_symname_map, and the K85 (MEAS) list draws graphics and names them with the value of symname. The numbers 3 and 1 in the figure are the directions marked by dir.

#### 4.3.4 Points
points is a list of pin points, each pin point can include multiple function points and BIT bits, the format is:
- name: pin point or function name
- description: pinpoint or function description
- x: optional, it is valid when background.type is image, which represents the x-coordinate of the graphic dot of the needle point.
- y: optional, if the background.type is image, it is valid, indicating the y-coordinate of the graphic dot of the needle point.
- list: list of function points or BIT bits. It is recommended to leave it empty, and configure it on the operation interface after importing it into the needle point management.

**Function point: refers to the combination of multiple BITs. Function points support nesting, which means that the combination can include function points**</br>
**BIT: The BIT of the controller to switch**


```js
{
  "name": "WL_HOST_WAKE_CONN_TO_DM01",
  "description": "WL_HOST_WAKE_CONN_TO_DM01 MEASUREMEANT",
  "x": 100,
  "y": 100,
  "list": [
    {
      {
        "name": "U42(MEAS)",
        "description": "U42 MEASUREMEANT",
        "list":[
          {
            "bit2": 0
          }, 
        ]
      },
      {
          "bit4": 0
      },      
    }
  ]
}
```
##### 4.3.2.1 When The Needle Point Has Only A BIT Bit

```js
{
  "name": "WL_HOST_WAKE_CONN_TO_DM01",
  "description": "WL_HOST_WAKE_CONN_TO_DM01 MEASUREMEANT",
  "list": [
    {
      {
          "bit4": 0
      },
      {
          "bit3": 0
      },
      {
          "bit2": 0
      },
      {
          "bit1": 1
      }
    }
  ]
}
```

##### 4.3.2.2 The Case Where The Pin Point Contains The Function Point And The BIT Bit
```js
{
  "name": "WL_HOST_WAKE_CONN_TO_DM01",
  "description": "WL_HOST_WAKE_CONN_TO_DM01 MEASUREMEANT",
  "list": [
    {
      {
        "name": "U42(MEAS)",
        "description": "U42 MEASUREMEANT",
        "list":[
          {
            "bit2": 0
          }, 
          {
            "bit3": 0
          }, 
        ]
      },
      {
        "bit4": 0
      },
      {
        "bit5": 1
      },          
    }
  ]
}
```

##### 4.3.2.3 Needle Point Contains Function Point Nesting
```js
{
  "name": "WL_HOST_WAKE_CONN_TO_DM01",
  "description": "WL_HOST_WAKE_CONN_TO_DM01 MEASUREMEANT",
  "list": [
    {
      {
        "name": "U42(MEAS)",
        "description": "U42 MEASUREMEANT",
        "list":[
          {
            "bit2": 0
          }, 
          {
            "bit3": 0
          }, 
        ]
      },
      {
        "name": "P_GPIO16_TO_DM01_RST",
        "description": "P_GPIO16_TO_DM01RESET",
        "list": [
          {
              "bit52": 0
          },
          {
              "bit37": 0
          },
          "U42(MEAS)",   //引用U42(MEAS)功能点
          {
              "bit38": 0
          },
          {
              "bit53": 0
          }
        ]
      }
      {
        "bit4": 0
      },      
    }
  ]
}
```

### 4.4 Write Pinpoint Table File
Optional, only need to be written when background.type is table.
The table is in csv format, and the rules are as follows:

```csv
aa1, aa2,...,NET,bb1,bb2,NET,bb1,bb2...
```

- NET is a fixed column with a fixed name of NET and cannot be deleted.
- There are two columns in the middle of NET, the names can be modified, such as bb1, bb2 or cc1, cc2
- There must be N columns (at least one column) in front of the first Net column, which is used to represent the needle point column. When the value of this column is PT_SHOW, it means that the needle point needs to be displayed (generated), and the name of the needle point is the name of Net + "TO" + the name of the table in this list (the generated name is consistent with the pin point name described in the config.json file); if it is empty or other values, the pin point will not be displayed (generated).
- There must be N columns (at least one column) after the second Net column, which is used to represent the needle point column. When the value of this column is PT_SHOW, it means that the needle point needs to be displayed (generated), and the name of the needle point is the name of Net +"TO"+Name of the table in this column; empty or other values do not display (generate) pinpoints.

csv example:
```csv
ZYNQ,SCOPE,DMM,DM01,NET,J1300,J1300,NET,DM01,DMM,SCOPE,ZYNQ
,,,,,1,2,BT_HOST_WAKE_CONN,PT_SHOW,,,
,,,,,3,4,BT_REG_ON_CONN,PT_SHOW,,,
,,,,,5,6,BT_DEV_WAKE_CONN,PT_SHOW,,,
,,,,,7,8,,,,,
PT_SHOW,,,PT_SHOW,SOC_TO_WLAN_PERST_N_CONN,9,10,SOC_TO_BT_UART_TXD_CONN,,,PT_SHOW,
PT_SHOW,PT_SHOW,,,WLAN_TO_SOC_CLKREQ_N_CONN,11,12,SOC_TO_BT_UART_RTS_CONN,,,PT_SHOW,
,,,PT_SHOW,WLAN_TO_SOC_WAKE_N_CONN,13,14,SOC_TO_BT_UART_CTS_CONN,,,PT_SHOW,
,,,,,15,16,SOC_TO_BT_UART_RXD_CONN,,,PT_SHOW,
,,,,,17,18,,,,,
```

The effect is as shown in the figure:
![point_4](./resource/en/v2.3/point_table_name_rule.png)

### 4.5 Import Needle Point Information
- Import graphic pinpoint configuration: After writing the pinpoint configuration, save it as config.json, click **Import pinpoint information** (as shown in Figure 1), select the config.json file you just saved, and click **Open**.
![point_4](./resource/en/v2.3/point_import_config.png)

- Import graphic table configuration: After writing the pin point configuration, save it as config.json, save the pin point table file as table.csv, then compress config.json and table.csv into config.zip, click ** to import Needle point information** (as shown in Figure 1), select the config.zip file just now, and click **Open**.

### 4.6 Configure Pinpoint Information
#### 4.6.1 Select The Pin Point That Needs To Be Configured
- As shown in Figure 1, select the pin point with configuration and enter the pin point configuration.
- As shown in Figure 2, enter the needle point name, and after querying the needle point information, as shown in Figure 3, click **Edit** to enter the needle point configuration.![point_4](./resource/en/v2.3/point_select_config_point.png)

#### 4.6.2 Configured Pin Point
- As shown in Figure 1, modify the needle point name.
- As shown in Figure 2, modify the pinpoint description.
- As shown in Figure 3, click **Add** to select the function point or BIT to be added.![point_4](./resource/en/v2.3/point_edit_point.png)

##### 4.6.2.1 Configure The Bits Of The BIT
Click **Add**, after selecting the BIT bit.
- As shown in Figure 1, modify the name of the BIT bit.
- As shown in Figure 2, modify the value of the BIT bit.![point_4](./resource/en/v2.3/point_edit_bit.png)

* A reference means that a configured function point can be referenced.

##### 4.6.2.2 Configured Function Points
- As shown in Figure 1, click **Edit** to enter the function point configuration interface, the interface operation is consistent with the needle point configuration.
![point_4](./resource/en/v2.3/point_edit_function.png)


### 4.7 Export Pin Point Information
Click **Export points**, select the export path, and the system exports different configuration files according to the type of graph: the graph view exports the config.json file, and the table view exports the config.zip file.

### 4.7 Reset To Default
Click **Reset to default** to restore the default graphics view configuration.
